@@ -12,7 +12,7 @@ Dancer::Plugin::Database - easy database connections for Dancer applications
 
 =cut
 
-our $VERSION = '1.30_01';
+our $VERSION = '1.42_01';
 
 my $settings = undef;
 
@@ -65,7 +65,7 @@ register database => sub {
     $handle = $handles{$pid_tid}{$handle_key} || {};
     
     if ($handle->{dbh}) {
-        if ($conn_details->{connection_check_threshold} &&
+        if ($handle->{dbh}{Active} && $conn_details->{connection_check_threshold} &&
             time - $handle->{last_connection_check}
             < $conn_details->{connection_check_threshold}) 
         {
@@ -78,7 +78,7 @@ register database => sub {
                 Dancer::Logger::debug(
                     "Database connection went away, reconnecting"
                 );
-                if ($handle->{dbh}) { $handle->{dbh}->disconnect; }
+                if ($handle->{dbh}) { eval { $handle->{dbh}->disconnect } }
                 return $handle->{dbh}= _get_connection($conn_details);
 
             }
@@ -103,10 +103,13 @@ sub _get_connection {
 
     # Assemble the DSN:
     my $dsn;
+    my $driver;
     if ($settings->{dsn}) {
         $dsn = $settings->{dsn};
+        ($driver) = $dsn =~ m{dbi:([^:]+)};
     } else {
         $dsn = "dbi:" . $settings->{driver};
+        $driver = $settings->{driver};
         my @extra_args;
 
         # DBD::SQLite wants 'dbname', not 'database', so special-case this
@@ -115,7 +118,7 @@ sub _get_connection {
         # things easier for our users by handling this for them):
         # (DBD::SQLite will support 'database', too, as of 1.32 when it's
         # released)
-        if ($settings->{driver} eq 'SQLite' 
+        if ($driver eq 'SQLite' 
             && $settings->{database} && !$settings->{dbname}) {
             $settings->{dbname} = delete $settings->{database};
         }
@@ -141,7 +144,10 @@ sub _get_connection {
             mysql  => 'mysql_enable_utf8',
             Pg     => 'pg_enable_utf8',
         );
-        if (my $param = $param_for_driver{ $settings->{driver} }) {
+
+        my $param = $param_for_driver{$driver};
+
+        if ($param && !$settings->{dbi_params}{$param}) {
             Dancer::Logger::debug(
                 "Adding $param to DBI connection params to enable UTF-8 support"
             );
@@ -186,7 +192,7 @@ sub _get_connection {
 sub _check_connection {
     my $dbh = shift;
     return unless $dbh;
-    if (my $result = $dbh->ping) {
+    if ($dbh->{Active} && (my $result = $dbh->ping)) {
         if (int($result)) {
             # DB driver itself claims all is OK, trust it:
             return 1;
@@ -295,6 +301,11 @@ no-op query against the database if not.  If the connection has gone away, a new
 connection will be obtained and returned.  This avoids any problems for
 a long-running script where the connection to the database might go away.
 
+Care is taken that handles are not shared across processes/threads, so this
+should be thread-safe with no issues with transactions etc.  (Thanks to Matt S
+Trout for pointing out the previous lack of thread safety.  Inspiration was
+drawn from DBIx::Connector.)
+
 =head1 CONFIGURATION
 
 Connection details will be taken from your Dancer application config file, and
@@ -305,6 +316,7 @@ should be specified as, for example:
             driver: 'mysql'
             database: 'test'
             host: 'localhost'
+            port: 3306
             username: 'myusername'
             password: 'mypassword'
             connection_check_threshold: 10
@@ -445,7 +457,9 @@ This module is developed on Github at:
 
 L<http://github.com/bigpresh/Dancer-Plugin-Database>
 
-Feel free to fork the repo and submit pull requests!
+Feel free to fork the repo and submit pull requests!  Also, it makes sense to 
+L<watch the repo|https://github.com/bigpresh/Dancer-Plugin-Database/toggle_watch> 
+on GitHub for updates.
 
 Feedback and bug reports are always appreciated.  Even a quick mail to let me
 know the module is useful to you would be very nice - it's nice to know if code
@@ -469,6 +483,14 @@ Carlos Sosa
 
 Matt S Trout
 
+Matthew Vickers
+
+Christian Walde
+
+Alberto Simões
+
+James Aitken (LoonyPandora)
+
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-dancer-plugin-database at rt.cpan.org>, or through
@@ -527,7 +549,7 @@ L<Dancer>
 
 L<DBI>
 
-
+L<Dancer::Plugin::SimpleCRUD>
 
 =cut
 
